# AdminIPLimiter - (c) 2018 Curlybracket, Inc. All Rights Reserved.
# This code cannot be redistributed without permission from curlybracket.co.jp
# For more information, consult your AdminIPLimiter license.
#
package AdminIPLimiter;

use strict;
use warnings;
use utf8;
use Carp;
use MT::Plugin;
use base qw( MT::Plugin );
use MT;

our $PLUGIN_NAME = 'AdminIPLimiter(Trial)';
our $VERSION = '1.00';

my $plugin = AdminIPLimiter->new({
    id => 'adminiplimiter',
    key => __PACKAGE__,
    name => "<__trans phrase='$PLUGIN_NAME'>",
    version => $VERSION,
    description => '<__trans phrase="This plugin manages whether to access the administration screen for each IP address.">',
    author_name => 'Curlybracket, Inc.',
    author_link => 'https://www.curlybracket.co.jp',
    l10n_class => 'AdminIPLimiter::L10N',
    system_config_template => 'system_config.tmpl',
    settings => new MT::PluginSettings([
        ['enabled', { Default => 0 }],
        ['permitted', { Default => q{# There are three formats for specifying IP Address as follows.
# Please set them one by one for each line.
# Each format can be mixed.
# Lines beginning with # are comments.
#
# 1. Specify one IP Address
# 192.168.1.1
#
# 2. Specify the IP Address range
# 192.168.1.1-192.168.1.100
#
# 3. Specify in CIDR notation
# 192.168.1.0/24
}
            }],
        ['denied', { Default => q{# There are three formats for specifying IP Address as follows.
# Please set them one by one for each line.
# Each format can be mixed.
# Lines beginning with # are comments.
#
# 1. Specify one IP Address
# 192.168.1.1
#
# 2. Specify the IP Address range
# 192.168.1.1-192.168.1.100
#
# 3. Specify in CIDR notation
# 192.168.1.0/24
}
            }],
        ['redirect_url', { Default => '' }],
        ['error_page_file_path', { Default => '' }],
    ]),
    registry => {
        applications => {
            cms => {
                methods => {
                  adminiplimiter_verify => '$AdminIPLimiter::AdminIPLimiter::CMS::verify',
                },
            },
        },
        callbacks =>  {
            init_app => {
                priority => 10,
                handler => \&init_app,
            },
            'save_config_filter.AdminIPLimiter' => \&save_config_filter,
        },
    },
});

MT->add_plugin($plugin);

sub instance { $plugin; }

sub doLog {
    my ($msg) = @_; 
    return unless defined($msg);

    require MT::Log;
    my $log = MT::Log->new;
    $log->message($msg) ;
    $log->save or die $log->errstr;
}

sub load_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    $plugin->SUPER::load_config($param, $scope);
    my $app = $plugin->app;
    if($app->isa('MT::App::CMS')) {
        $param->{target_ip} = $app->remote_ip;
    }
    return;
}

sub save_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    delete $plugin->{'permitted_cidr'};
    delete $plugin->{'denied_cidr'};
    my $app = $plugin->app;
    if($app->can('reboot')) {
        $app->reboot;
    }
    return $plugin->SUPER::save_config($param, $scope);
}

sub apply_default_settings {
    my $plugin = shift;
    my ( $data, $scope ) = @_;
    if($scope eq 'system') {
        my $s = $plugin->settings;
        my $defaults = $s->defaults($scope);
        if ( $s && $defaults ) {
            foreach ( keys %$defaults ) {
                if(!exists $data->{$_}) {
                    if($_ eq 'permitted' || $_ eq 'denied') {
                        $data->{$_} = $plugin->translate($defaults->{$_});
                    }
                    else {
                        $data->{$_} = $defaults->{$_};
                    }
                }
            }
        }
    }
    else {
        $plugin->SUPER::apply_default_settings(@_);
    }
}

sub enabled {
    my $plugin = shift;
    my %plugin_param;
    $plugin->load_config(\%plugin_param, 'system');
    my $value = $plugin_param{enabled};
    return $value;
}

sub denied {
    my $plugin = shift;
    my %plugin_param;
    $plugin->load_config(\%plugin_param, 'system');
    my $value = $plugin_param{denied};
    return $value;
}


sub permitted {
    my $plugin = shift;
    my %plugin_param;
    $plugin->load_config(\%plugin_param, 'system');
    my $value = $plugin_param{permitted};
    return $value;
}

sub redirect_url {
    my $plugin = shift;
    my %plugin_param;
    $plugin->load_config(\%plugin_param, 'system');
    my $value = $plugin_param{redirect_url};
    return $value;
}

sub error_page_file_path {
    my $plugin = shift;
    my %plugin_param;
    $plugin->load_config(\%plugin_param, 'system');
    my $value = $plugin_param{error_page_file_path};
    return $value;
}

sub app {
    my $plugin = shift;
    return MT->instance;
}

sub get_cidr {
    my $plugin = shift;
    my ($which, $text, $no_cache) = @_;

    $which ||= 'permitted';
    croak $plugin->translate('Invalid arg, which')
        unless $which =~ m!(?:permitted|denied)!;

    my $cidr = $plugin->{"${which}_cidr"}; 
    if(!defined $cidr || $no_cache) {
        require Net::CIDR::Lite;
        $text ||= $plugin->$which;
        my @lines = split /\r?\n/, $text;
        @lines = grep { m/^(?!\s*#).*$/ }
                    map { s!^\s*!!; s!\s*$!!; $_ } @lines;
        @lines = grep { length $_ > 0 } @lines;
        if(@lines) {
            eval {
                $cidr = Net::CIDR::Lite->new();
                $cidr->add_any($_) for @lines;
            };
            if($@) {
                my $e = $@;
                croak $plugin->translate('Invalid settings: [_1]', $e);
            }
        }
        else {
            $cidr = 0;
        }

        if(!$no_cache) {
            $plugin->{"${which}_cidr"} = $cidr;
        }
    }

    return $cidr;
}

sub permitted_cidr {
    my $plugin = shift;
    my ($text, $no_cache) = @_;
    return $plugin->get_cidr('permitted', $text, $no_cache);
}

sub denied_cidr {
    my $plugin = shift;
    my ($text, $no_cache) = @_;
    return $plugin->get_cidr('denied', $text, $no_cache);
}

sub can_access {
    my $plugin = shift;
    my ($target_ip, $opt) = @_;

    croak $plugin->translate('Missing arg, target_ip')
        unless $target_ip;

    my $no_cache = defined $opt && $opt->{no_cache};
    my $permitted = defined $opt ? $opt->{permitted} : undef;
    my $denied = defined $opt ? $opt->{denied} : undef;
    my $ok = 0;

    require Net::CIDR::Lite;
    if(my $permitted_cidr = $plugin->permitted_cidr($permitted, $no_cache)) {
        if($permitted_cidr->find($target_ip)) {
            $ok = 1;
        }
    }
    else {
        $ok = 1;
    }

    if(my $denied_cidr = $plugin->denied_cidr($denied, $no_cache)) {
        if($denied_cidr->find($target_ip)) {
            $ok = 0;
        }
    }

    return $ok;
}

sub show_error {
    my $plugin = shift;
    my $app = $plugin->app;
    $app->response_code(403);
    $app->response_message('Forbidden');
    $app->send_http_header;
    $app->{no_print_body} = 1;
    my $content = <<HTML;
<html><head><title>Forbidden</title><head><body style="text-align:center;"><h1 style="font-size:1600%;margin:0;">403</h1><h2 style="font-size:400%;margin:0">Forbidden</h2></body></html>
HTML
    require MT::FileMgr;
    my $fmgr = MT::FileMgr->new('Local');
    my $error_page_file_path = $plugin->error_page_file_path || '';
    if($error_page_file_path && $fmgr->exists($error_page_file_path)) {
        $content = $fmgr->get_data($error_page_file_path);
    }
    $app->print_encode($content);
    return;
}

sub init_app {
    my ( $cb, $app ) = @_;

    return 1 if $app->config->DisableAdminIPLimiter;
    my $plugin = $app->component('AdminIPLimiter')
        or return 1;
    return 1 unless $plugin->enabled;
    return 1 if $app->{adminiplimiter_init};

    require MT::App::CMS;
    no strict 'refs';
    no warnings 'redefine';
    my $saved_validate_request_params = \&MT::App::CMS::validate_request_params;
    my $_validate_request_params = sub {
        my $app = shift;
        if(!$app->config->DisableAdminIPLimiter && $plugin->enabled) {
            my $remote_ip = $app->remote_ip;
            # printf STDERR "remote_id: %s\n", $remote_ip;
            if(!$plugin->can_access($remote_ip, { no_cache => 0 })) {
                unless($app->config->DisableAdminIPLimiterLog) {
                    my $user_agent = $ENV{HTTP_USER_AGENT} || '';
                    $app->log({
                        message => $plugin->translate('Client denied: [_1]', $remote_ip),
                        level => MT::Log::SECURITY(),
                        class => 'system',
                        category => 'adminiplimiter',
                        metadata => $user_agent,
                    });
                }
                $app->{redirect} = $plugin->redirect_url
                    or $plugin->show_error;
                die $app->translate('Prohibited'); 
            }
        }
        return &$saved_validate_request_params($app, @_);
    };
    *MT::App::CMS::validate_request_params = $_validate_request_params;

    $app->{adminiplimiter_init} = 1;

    return 1;
}

sub save_config_filter {
    my ( $cb, $plugin, $data, $scope ) = @_;

    return 1 unless defined $scope && $scope eq 'system';
    
    foreach my $column (qw/permitted denied/) {
        if($data->{$column}) {
            eval {
                my $cidr = $plugin->get_cidr($column, $data->{$column}, 1);
            };
            if($@) {
                my $e = $@;
                if($e =~ m!^(.+?)( at .+? line \d+)(.*)$!s) {
                    $e = $1;
                }
                my $prefix = $column eq 'permitted'
                    ? $plugin->translate('Permitted IP Address')
                    : $plugin->translate('Denied IP Address');
                return $plugin->error($prefix . ': ' . $e);
            }
        }
    }

    return 1;
}

1;
